@@ -31,15 +31,16 @@ import org.apache.empire.data.Column;
 import org.apache.empire.db.DBColumn;
 import org.apache.empire.db.DBColumnExpr;
 import org.apache.empire.db.DBCommand;
-import org.apache.empire.db.DBDatabaseDriver;
-import org.apache.empire.db.DBDriverFeature;
+import org.apache.empire.db.DBContext;
 import org.apache.empire.db.DBReader;
 import org.apache.empire.db.DBRecordData;
 import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.expr.order.DBOrderByExpr;
+import org.apache.empire.dbms.DBMSHandler;
+import org.apache.empire.dbms.DBMSFeature;
 import org.apache.empire.exceptions.InternalException;
 import org.apache.empire.exceptions.InvalidArgumentException;
-import org.apache.empire.exceptions.MiscellaneousErrorException;
+import org.apache.empire.exceptions.UnspecifiedErrorException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
@@ -53,7 +54,7 @@ import org.slf4j.LoggerFactory;
 
 public class BeanListPageElement<T> extends ListPageElement<T> implements ListItemSelection
 {
-    private static final long   serialVersionUID     = 1L;
+    // *Deprecated* private static final long serialVersionUID = 1L;
 
     private static final Logger log                  = LoggerFactory.getLogger(BeanListPageElement.class);
     
@@ -76,7 +77,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
      */
     public static class BeanListTableInfo extends ListTableInfo
     {
-        private static final long serialVersionUID = 1L;
+        // *Deprecated* private static final long serialVersionUID = 1L;
 
         private DBCommand         queryCmd         = null;
 
@@ -247,6 +248,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     public void initItems(DBCommand queryCmd, DBCommand countCmd, int pageSize)
     {
         clearItems();
+        DBContext context = getDBContext(rowset);
         // Init List Table Info
         BeanListTableInfo lti = (BeanListTableInfo) getTableInfo();
         lti.setQueryCmd(queryCmd);
@@ -254,7 +256,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         { // Negative count means: loadItems should load all items.
             countCmd.clearSelect();
             countCmd.select(rowset.count());
-            int count = rowset.getDatabase().querySingleInt(countCmd.getSelect(), countCmd.getParamValues(), 0, getConnection(rowset));
+            int count = context.getUtils().querySingleInt(countCmd, 0);
             lti.init(count, pageSize);
         }
         else
@@ -319,7 +321,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     {
         // DBReader
         BeanListTableInfo lti = (BeanListTableInfo) getTableInfo();
-        DBReader r = new DBReader();
+        DBReader r = new DBReader(getDBContext(rowset));
         try
         { // Check command
             DBCommand queryCmd = lti.getQueryCmd();
@@ -354,10 +356,10 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
                 skipRows = position;
                 // constraint
                 queryCmd.clearLimit();
-                DBDatabaseDriver driver = queryCmd.getDatabase().getDriver(); 
-                if (driver.isSupported(DBDriverFeature.QUERY_LIMIT_ROWS))
+                DBMSHandler dbms = r.getContext().getDbms(); 
+                if (dbms.isSupported(DBMSFeature.QUERY_LIMIT_ROWS))
                 {   // let the database limit the rows
-                    if (driver.isSupported(DBDriverFeature.QUERY_SKIP_ROWS))
+                    if (dbms.isSupported(DBMSFeature.QUERY_SKIP_ROWS))
                     {   // let the database skip the rows
                         queryCmd.skipRows(skipRows);
                         skipRows = 0;
@@ -367,7 +369,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
             }
 
             // DBReader.open must always be surrounded with a try {} finally {} block!
-            r.open(queryCmd, getConnection(queryCmd));
+            r.open(queryCmd);
 
             // get position from the session
             if (skipRows>0)
@@ -574,7 +576,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
 
     protected void generateIdParams(DBRowSet rowset, List<?> items)
     {
-        DBColumn[] keyCols = rowset.getKeyColumns();
+        Column[] keyCols = rowset.getKeyColumns();
         if (keyCols == null)
             return; // No Primary Key!
         // generate all
@@ -589,20 +591,20 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         }
     }
 
-    protected Object[] getItemKey(DBColumn[] cols, Object item)
+    protected Object[] getItemKey(Column[] cols, Object item)
     {
         Object[] key = new Object[cols.length];
         for (int i = 0; i < cols.length; i++)
         {
             if (item instanceof DBRecordData)
             {
-                key[i] = ((DBRecordData) item).getValue(cols[i]);
+                key[i] = ((DBRecordData) item).get(cols[i]);
             }
             else
             { // Bean Property Name
                 String propName = cols[i].getBeanPropertyName();
                 if (propName == null || propName.length() == 0)
-                    throw new MiscellaneousErrorException("Invalid Bean Property Name");
+                    throw new UnspecifiedErrorException("Invalid Bean Property Name");
                 // Get Property value
                 try
                 {
@@ -630,15 +632,13 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         {
             DBColumn[] pk = rowset.getKeyColumns();
             DBColumnExpr keyExpr = pk[0];
-           
             for (int i=1; i<pk.length; i++)
             {
                 keyExpr = keyExpr.append(pk[i]);
             }
             
-            String[] keys = new String[items.size()];
+            Object[] keys = new Object[items.size()];
             int i = 0;
-
             for (Object[] item : items)
             {
                 keys[i++] = StringUtils.arrayToString(item, "");
