@@ -17,23 +17,30 @@
  * under the License.
  */
 package org.apache.empire.db;
+import java.beans.PropertyDescriptor;
 // XML
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.math.BigDecimal;
+import java.time.LocalDate;
+import java.time.LocalDateTime;
 import java.util.Collection;
 import java.util.Date;
 
 import org.apache.commons.beanutils.BeanUtils;
 import org.apache.commons.beanutils.PropertyUtils;
 import org.apache.empire.commons.ObjectUtils;
+import org.apache.empire.commons.Options;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.ColumnExpr;
 import org.apache.empire.data.RecordData;
+import org.apache.empire.db.context.DBContextAware;
 import org.apache.empire.db.exceptions.FieldIllegalValueException;
 import org.apache.empire.exceptions.BeanPropertySetException;
 import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.ItemNotFoundException;
+import org.apache.empire.exceptions.PropertyReadOnlyException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Document;
@@ -41,41 +48,70 @@ import org.w3c.dom.Element;
 
 
 /**
- * This interface defines for the classes DDRecordSet and DBRecord.
- * <P>
- * 
- *
+ * This class provides access to the fields of one data-row of a table, view or query
+ * The fields can be accessed either by Column or by index
+ * There are various accessor functions for many data types.
+ * The field values are converted to the desired type if possible.
+ * The field values can be transferred to a classical Java-bean (aka Pojo)
  */
-public abstract class DBRecordData extends DBObject
-	implements RecordData
+public abstract class DBRecordData extends DBObject 
+	implements DBContextAware, RecordData
 {
-    private final static long serialVersionUID = 1L;
-  
+    // *Deprecated* private static final long serialVersionUID = 1L;
+
     // Logger
     private static final Logger log = LoggerFactory.getLogger(DBRecordData.class);
-    
+
     // Field Info
     @Override
-    public abstract int     getFieldCount();
+    public abstract int getFieldCount();
+
     @Override
-    public abstract int  	getFieldIndex(ColumnExpr column);
+    public abstract int getFieldIndex(ColumnExpr column);
+
     @Override
-    public abstract int  	getFieldIndex(String column);
-    // Column lookup
-    @Override
-    public abstract ColumnExpr getColumnExpr(int i);
+    public abstract int getFieldIndex(String column);
+
     // xml
-    public abstract int     addColumnDesc(Element parent);
-    public abstract int     addRowValues (Element parent);
+    public abstract int addXmlMeta(Element parent);
+    public abstract int addXmlData(Element parent);
     public abstract Document getXmlDocument();
+
     // others
-    public abstract void    close();
+    public abstract void close();
 
     /**
      * Returns a value based on an index.
      */
     @Override
-    public abstract Object  getValue(int index);
+    public abstract Object getValue(int index);
+    
+    /**
+     * @Deprecated Renamed to get(...)   
+     */
+    @Deprecated
+    public Object getValue(ColumnExpr column)
+    {
+        return get(column);
+    }
+
+    /**
+     * @Deprecated Renamed to get(...)   
+     */
+    @Deprecated
+    public final <T> T getValue(Column column, Class<T> returnType)
+    {
+        return get(column, returnType);
+    }
+
+    /**
+     * @Deprecated Renamed to get(...)   
+     */
+    @Deprecated
+    public final Object[] getValues(ColumnExpr... columns)
+    {
+        return getArray(columns);
+    }
     
     /**
      * Returns a data value for the desired column .
@@ -84,7 +120,7 @@ public abstract class DBRecordData extends DBObject
      * @return the record value
      */
     @Override
-    public final Object getValue(ColumnExpr column)
+    public final Object get(ColumnExpr column)
     {
         int index = getFieldIndex(column);
         if (index<0)
@@ -92,13 +128,24 @@ public abstract class DBRecordData extends DBObject
         return getValue(index);
     }
 
+    /**
+     * Returns the value of a field as an object of a given (wrapper)type
+     * @param column the column for which to retrieve the value
+     * @param returnType the type of the returned value
+     * @return the value
+     */
+    public final <T> T get(Column column, Class<T> returnType)
+    {
+        return ObjectUtils.convert(returnType, get(column));
+    }
+
     /**
      * Returns an array of values for the given column expressions
      * 
      * @param column the column expressions
      * @return the corresponding record values
      */
-    public final Object[] getValues(ColumnExpr[] columns)
+    public final Object[] getArray(ColumnExpr... columns)
     {
         Object[] values = new Object[columns.length];
         for (int i=0; i<columns.length; i++)
@@ -120,9 +167,7 @@ public abstract class DBRecordData extends DBObject
      */
     public int getInt(int index)
     {
-        // Get Integer value
-        Object value = getValue(index);
-        return ObjectUtils.getInteger(value);
+        return ObjectUtils.getInteger(getValue(index));
     }
     
     /**
@@ -146,9 +191,7 @@ public abstract class DBRecordData extends DBObject
      */
     public long getLong(int index)
     {
-        // Get Integer value
-        Object value = getValue(index);
-        return ObjectUtils.getLong(value);
+        return ObjectUtils.getLong(getValue(index));
     }
     
     /**
@@ -172,9 +215,7 @@ public abstract class DBRecordData extends DBObject
      */
     public double getDouble(int index)
     {
-        // Get Double value
-        Object value = getValue(index);
-        return ObjectUtils.getDouble(value);
+        return ObjectUtils.getDouble(getValue(index));
     }
 
     /**
@@ -198,9 +239,7 @@ public abstract class DBRecordData extends DBObject
      */
     public BigDecimal getDecimal(int index)
     {
-        // Get Double value
-        Object value = getValue(index);
-        return ObjectUtils.getDecimal(value);
+        return ObjectUtils.getDecimal(getValue(index));
     }
 
     /**
@@ -224,9 +263,7 @@ public abstract class DBRecordData extends DBObject
      */
     public boolean getBoolean(int index)
     {
-        // Get Boolean value
-        Object value = getValue(index);
-        return ObjectUtils.getBoolean(value);
+        return ObjectUtils.getBoolean(getValue(index));
     }
     
     /**
@@ -237,7 +274,9 @@ public abstract class DBRecordData extends DBObject
      * @return the value
      */
     public final boolean getBoolean(ColumnExpr column)
-    { return getBoolean(getFieldIndex(column)); }
+    { 
+        return getBoolean(getFieldIndex(column));
+    }
     
     /**
      * Returns a data value identified by the column index.
@@ -248,9 +287,7 @@ public abstract class DBRecordData extends DBObject
      */
     public String getString(int index)
     {
-        // Get Integer value
-        Object value = getValue(index);
-        return StringUtils.toString(value);
+        return ObjectUtils.getString(getValue(index));
     }
 
     /**
@@ -272,11 +309,9 @@ public abstract class DBRecordData extends DBObject
      * @param index index of the column
      * @return the value
      */
-    public Date getDateTime(int index)
+    public final Date getDateTime(int index)
     {
-        // Get DateTime value
-        Object value = getValue(index);
-        return ObjectUtils.getDate(value);
+        return ObjectUtils.getDate(getValue(index));
     }
     
     /**
@@ -291,6 +326,53 @@ public abstract class DBRecordData extends DBObject
         return getDateTime(getFieldIndex(column));
     }
 
+    /**
+     * Returns a data value identified by the column index.
+     * The data value is converted to a Date if necessary.
+     * 
+     * @param index index of the column
+     * @return the value
+     */
+    public final LocalDate getLocalDate(int index)
+    {
+        return ObjectUtils.getLocalDate(getValue(index));
+    }
+    
+    /**
+     * Returns a data value for the desired column.
+     * The data value is converted to a Date if necessary.
+     * 
+     * @param column identifying the column
+     * @return the value
+     */
+    public final LocalDate getLocalDate(ColumnExpr column)
+    {
+        return getLocalDate(getFieldIndex(column));
+    }
+
+    /**
+     * Returns a data value identified by the column index.
+     * The data value is converted to a Date if necessary.
+     * 
+     * @param index index of the column
+     * @return the value
+     */
+    public final LocalDateTime getLocalDateTime(int index)
+    {
+        return ObjectUtils.getLocalDateTime(getValue(index));
+    }
+    
+    /**
+     * Returns a data value for the desired column.
+     * The data value is converted to a Date if necessary.
+     * 
+     * @param column identifying the column
+     * @return the value
+     */
+    public final LocalDateTime getLocalDateTime(ColumnExpr column)
+    {
+        return getLocalDateTime(getFieldIndex(column));
+    }
 
     /**
      * Returns the value of a field as an enum
@@ -305,7 +387,7 @@ public abstract class DBRecordData extends DBObject
         if (isNull(index))
             return null;
         // convert
-        ColumnExpr col = getColumnExpr(index);
+        ColumnExpr col = getColumn(index);
         try {
             // Convert to enum, depending on DataType
             boolean numeric = col.getDataType().isNumeric();
@@ -349,26 +431,6 @@ public abstract class DBRecordData extends DBObject
         }
         return getEnum(getFieldIndex(column), (Class<T>)enumType);
     }
-
-    /**
-     * Returns the value of a field as an object of a given (wrapper)type
-     * @param index index of the field
-     * @return the value
-     */
-    public final <T> T getAs(int index, Class<T> wrapperType)
-    {
-        return ObjectUtils.convert(wrapperType, getValue(index));
-    }
-
-    /**
-     * Returns the value of a field as an object of a given (wrapper)type
-     * @param column the column for which to retrieve the value
-     * @return the value
-     */
-    public final <T> T getAs(Column column, Class<T> wrapperType)
-    {
-        return ObjectUtils.convert(wrapperType, getValue(column));
-    }
     
     /**
      * Checks whether or not the value for the given column is null.
@@ -379,7 +441,7 @@ public abstract class DBRecordData extends DBObject
     @Override
     public boolean isNull(int index)
     {
-        return (getValue(index) == null);
+        return ObjectUtils.isEmpty(getValue(index));
     }
 
     /**
@@ -393,7 +455,54 @@ public abstract class DBRecordData extends DBObject
     {
         return isNull(getFieldIndex(column));
     }
+    
+    /**
+     * Returns the value of a column as a formatted text
+     * This converts the value to a string if necessary and performs an options lookup
+     * To customize conversion please override convertToString()
+     * @param column the column for which to get the formatted value
+     * @return the formatted value
+     */
+    public String getText(ColumnExpr column)
+    {   
+        String text;
+        Object value = get(column);
+        // check options first
+        Options options = column.getOptions();
+        if (options!=null && options.has(value))
+        {   // lookup option
+            text = options.get(value);
+        }
+        else if (value instanceof String)
+        {   // we already have a string
+            text = (String)value;
+        }
+        else if (ObjectUtils.isEmpty(value))
+        {   // empty
+            value = column.getAttribute(Column.COLATTR_NULLTEXT);
+            text = (value!=null ? value.toString() : StringUtils.EMPTY);
+        }
+        else
+        {   // format value
+            text = formatValue(column, value);
+            if (text== null)
+                text = StringUtils.EMPTY; 
+        }
+        // done
+        return text;
+    }
 
+    /**
+     * Convert a non-string value to a string
+     * @param column the column expression 
+     * @param value the value to format
+     * @return the formatted string
+     */
+    protected String formatValue(ColumnExpr column, Object value)
+    {
+        return ObjectUtils.getString(value);
+    }
+    
     /**
      * Set a single property value of a java bean object used by readProperties.
      */
@@ -407,30 +516,34 @@ public abstract class DBRecordData extends DBObject
                 throw new InvalidArgumentException("bean", bean);
             if (StringUtils.isEmpty(property))
                 throw new InvalidArgumentException("property", property);
-            /*
             if (log.isTraceEnabled())
-                log.trace(bean.getClass().getName() + ": setting property '" + property + "' to " + String.valueOf(value));
-            */
+                log.trace("{}: setting property '{}' to {}", bean.getClass().getName(), property, value);
             /*
             if (value instanceof Date)
             {   // Patch for date bug in BeanUtils
                 value = DateUtils.addDate((Date)value, 0, 0, 0);
             }
             */
-            @SuppressWarnings("unchecked")
-            Class<Enum<?>> enumType = (Class<Enum<?>>)column.getAttribute(Column.COLATTR_ENUMTYPE);
-            if (enumType!=null && value!=null)
-            {   // value to enum
-                value = ObjectUtils.getEnum(enumType, value);
-            }
             // Set Property Value
             if (value!=null)
-            {   // Bean utils will convert if necessary
+            {   // Convert to enum
+                Class<Enum<?>> enumType = column.getEnumType();
+                if (enumType!=null)
+                    value = ObjectUtils.getEnum(enumType, value);
+                // Bean utils will convert if necessary
                 BeanUtils.setProperty(bean, property, value);
             }
             else
             {   // Don't convert, just set
-                PropertyUtils.setProperty(bean, property, null);
+                PropertyDescriptor pd = PropertyUtils.getPropertyDescriptor(bean, property);
+                if (pd==null)
+                    return; // No such property
+                // get the write method
+                final Method method = PropertyUtils.getWriteMethod(pd);
+                if (method == null)
+                    throw new PropertyReadOnlyException(property);
+                // invoke
+                method.invoke(bean, value);
             }
           // IllegalAccessException
         } catch (IllegalAccessException e)
@@ -461,8 +574,8 @@ public abstract class DBRecordData extends DBObject
         // Add all Columns
         int count = 0;
         for (int i = 0; i < getFieldCount(); i++)
-        { // Check Property
-            ColumnExpr column = getColumnExpr(i);
+        {   // Check Property
+            ColumnExpr column = getColumn(i);
             if (ignoreList != null && ignoreList.contains(column))
                 continue; // ignore this property
             // Get Property Name
@@ -479,7 +592,6 @@ public abstract class DBRecordData extends DBObject
      * 
      * @return the number of bean properties set on the supplied bean
      */
-    @Override
     public final int setBeanProperties(Object bean)
     {
         return setBeanProperties(bean, null);
